/*
 * Copyright (c) 1999 by Sun Microsystems, Inc.
 * All rights reserved.
 */

#pragma ident	"@(#)getpeerinfo_server.c	6.1	99/11/16 SMI; TSOL 2.x"


#include <errno.h>
#include <stdio.h>
#include <string.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <bsm/audit.h>
#include <tsix/t6attrs.h>
#include <tsol/priv.h>

#include "getpeerinfo.h"

extern int _debug;
extern void _msgout(char *msg);


peerinfo_res *
getpeerinfo_1(argp, rqstp)
	peerinfo_arg *argp;
	struct svc_req *rqstp;
{
	static peerinfo_res	result;
	au_peerinfo_t		pinfo;
	priv_set_t		*pset;
	char			buf[80];
	struct netbuf		*netbuf;

	(void) memset(&result, 0, sizeof (peerinfo_res));
	pinfo.peer_pid = argp->pid;
	pinfo.peergroupinfo =
	    (au_peergroupinfo_t *)&(result.peerinfo_res_u.info.grpinfo);
	pinfo.peermiscinfo =
	    (au_peermiscinfo_t *)&(result.peerinfo_res_u.info.prinfo);

	/* get the privs */
	if ((pset = (priv_set_t *)t6get_attr(T6_PRIVILEGES,
	    rqstp->rq_xprt->xp_tsol_incoming_attrsp)) == NULL) {
		result.rpcerrno = EINVAL;
		netbuf = svc_getrpccaller(rqstp->rq_xprt);
		(void) sprintf(buf,
		    "unable to get request privileges for %d %s",
		    pinfo.peer_pid,
		    inet_ntoa(((struct sockaddr_in *)netbuf->buf)->sin_addr));
		_msgout(buf);
		return (&result);
	}

	/*
	 * check that user has either PRIV_PROC_AUDIT_TCB or PROC_AUDIT_APPL
	 */
	if ((PRIV_ISASSERT(pset, PRIV_PROC_AUDIT_TCB) == 0) &&
	    (PRIV_ISASSERT(pset, PRIV_PROC_AUDIT_APPL) == 0)) {
		if (_debug) {
			netbuf = svc_getrpccaller(rqstp->rq_xprt);
			(void) sprintf(buf,
			    "%d %s does not have appropriate audit privilege "
			    "only groups returned",
			    pinfo.peer_pid,
			    inet_ntoa(((struct sockaddr_in *)netbuf->buf)->
			    sin_addr));
			_msgout(buf);
		}
		pinfo.peermiscinfo = (au_peermiscinfo_t *)NULL;
	}

	if (_debug) {
		netbuf = svc_getrpccaller(rqstp->rq_xprt);
		(void) sprintf(buf, "processing call for %d, %s",
		    pinfo.peer_pid,
		    inet_ntoa(((struct sockaddr_in *)netbuf->buf)->sin_addr));
		_msgout(buf);
	}

	if (auditon(A_GETPEERINFO, (caddr_t)&pinfo, 0) == -1) {
		result.rpcerrno = errno;
		netbuf = svc_getrpccaller(rqstp->rq_xprt);
		(void) sprintf(buf, "%d %s auditon() call failed [error: %s]",
		    pinfo.peer_pid,
		    inet_ntoa(((struct sockaddr_in *)netbuf->buf)->sin_addr),
		    strerror(errno));
		_msgout(buf);
		return (&result);
	}

	result.rpcerrno = 0;
	return (&result);
}


peerinfo_res *
getpeerinfo_2(argp, rqstp)
	peerinfo_arg2 *argp;
	struct svc_req *rqstp;
{
	static peerinfo_res	result;
	au_peerinfo_t		pinfo;
	priv_set_t		*pset;
	char			buf[80];
	struct netbuf		*netbuf;

	(void) memset(&result, 0, sizeof (peerinfo_res));
	pinfo.peer_pid = argp->pid;

	if ((argp->type == ALLINFO) || (argp->type == GRPINFO)) {
		pinfo.peergroupinfo =
		    (au_peergroupinfo_t *)&(result.peerinfo_res_u.info.grpinfo);
		pinfo.peermiscinfo = (au_peermiscinfo_t *)NULL;
	} else {
		pinfo.peergroupinfo = (au_peergroupinfo_t *)NULL;
	}

	if ((argp->type == ALLINFO) || (argp->type == AUDINFO)) {
		pinfo.peermiscinfo =
		    (au_peermiscinfo_t *)&(result.peerinfo_res_u.info.prinfo);

		/* get the privs */
		if ((pset = (priv_set_t *)t6get_attr(T6_PRIVILEGES,
		    rqstp->rq_xprt->xp_tsol_incoming_attrsp)) == NULL) {
			netbuf = svc_getrpccaller(rqstp->rq_xprt);
			result.rpcerrno = EINVAL;
			(void) sprintf(buf,
			    "unable to get request privileges for %d %s",
			    pinfo.peer_pid,
			    inet_ntoa(((struct sockaddr_in *)netbuf->buf)->
			    sin_addr));
			_msgout(buf);
			return (&result);
		}

		/* check that user has either PRIV_PROC_AUDIT_TCB */
		/* or PROC_AUDIT_APPL */
		if ((PRIV_ISASSERT(pset, PRIV_PROC_AUDIT_TCB) == 0) &&
		    (PRIV_ISASSERT(pset, PRIV_PROC_AUDIT_APPL) == 0)) {
			netbuf = svc_getrpccaller(rqstp->rq_xprt);
			result.rpcerrno = EACCES;
			(void) sprintf(buf,
			    "%d %s does not have appropriate audit privilege",
			    pinfo.peer_pid,
			    inet_ntoa(((struct sockaddr_in *)netbuf->buf)->
			    sin_addr));
			_msgout(buf);
			return (&result);
		}
	}

	if (_debug) {
		netbuf = svc_getrpccaller(rqstp->rq_xprt);
		(void) sprintf(buf, "processing call for %d, %s",
		    pinfo.peer_pid,
		    inet_ntoa(((struct sockaddr_in *)netbuf->buf)->sin_addr));
		_msgout(buf);
	}

	if (auditon(A_GETPEERINFO, (caddr_t)&pinfo, 0) == -1) {
		netbuf = svc_getrpccaller(rqstp->rq_xprt);
		result.rpcerrno = errno;
		(void) sprintf(buf, "%d %s auditon() call failed [error: %s]",
		    pinfo.peer_pid,
		    inet_ntoa(((struct sockaddr_in *)netbuf->buf)->sin_addr),
		    strerror(errno));
		_msgout(buf);
		return (&result);
	}

	result.rpcerrno = 0;
	return (&result);
}
